@@ -351,7 +351,6 @@ static const struct zebra_desc_table command_types[] = {
 	DESC_ENTRY(ZEBRA_BFD_DEST_REPLAY),
 	DESC_ENTRY(ZEBRA_REDISTRIBUTE_ROUTE_ADD),
 	DESC_ENTRY(ZEBRA_REDISTRIBUTE_ROUTE_DEL),
-	DESC_ENTRY(ZEBRA_VRF_UNREGISTER),
 	DESC_ENTRY(ZEBRA_VRF_ADD),
 	DESC_ENTRY(ZEBRA_VRF_DELETE),
 	DESC_ENTRY(ZEBRA_VRF_LABEL),
@@ -436,6 +435,9 @@ static const struct zebra_desc_table command_types[] = {
 	DESC_ENTRY(ZEBRA_SRV6_LOCATOR_DELETE),
 	DESC_ENTRY(ZEBRA_SRV6_MANAGER_GET_LOCATOR_CHUNK),
 	DESC_ENTRY(ZEBRA_SRV6_MANAGER_RELEASE_LOCATOR_CHUNK),
+	DESC_ENTRY(ZEBRA_SRV6_MANAGER_GET_LOCATOR),
+	DESC_ENTRY(ZEBRA_SRV6_MANAGER_GET_SRV6_SID),
+	DESC_ENTRY(ZEBRA_SRV6_MANAGER_RELEASE_SRV6_SID),
 	DESC_ENTRY(ZEBRA_ERROR),
 	DESC_ENTRY(ZEBRA_CLIENT_CAPABILITIES),
 	DESC_ENTRY(ZEBRA_OPAQUE_MESSAGE),
@@ -461,7 +463,8 @@ static const struct zebra_desc_table command_types[] = {
 	DESC_ENTRY(ZEBRA_TC_CLASS_DELETE),
 	DESC_ENTRY(ZEBRA_TC_FILTER_ADD),
 	DESC_ENTRY(ZEBRA_TC_FILTER_DELETE),
-	DESC_ENTRY(ZEBRA_OPAQUE_NOTIFY)
+	DESC_ENTRY(ZEBRA_OPAQUE_NOTIFY),
+	DESC_ENTRY(ZEBRA_SRV6_SID_NOTIFY)
 };
 #undef DESC_ENTRY
 
