@@ -193,14 +193,16 @@ extern int mgmt_fe_adapter_send_rpc_reply(uint64_t session_id, uint64_t txn_id,
  *     req_id: the req id for the edit message
  *     unlock: implicit-lock flag was set in the request
  *     commit: implicit-commit flag was set in the request
- *     xpath: the xpath of the data node that was created
- *     error: the error code, zero for successful request
+ *     created: true if the node was just created
+ *     xpath: the xpath of the data node that was created/updated
+ *     error: >0 LY_ERR, < 0 -errno
  *     errstr: the error string, if error is non-zero
  */
 extern int mgmt_fe_adapter_send_edit_reply(uint64_t session_id, uint64_t txn_id,
 					   uint64_t req_id, bool unlock,
-					   bool commit, const char *xpath,
-					   int16_t error, const char *errstr);
+					   bool commit, bool created,
+					   const char *xpath, int16_t error,
+					   const char *errstr);
 
 /**
  * Send an error back to the FE client using native messaging.
@@ -210,7 +212,7 @@ extern int mgmt_fe_adapter_send_edit_reply(uint64_t session_id, uint64_t txn_id,
  * Args:
  *	txn_id: the txn_id this error pertains to.
  *	short_circuit_ok: True if OK to short-circuit the call.
- *	error: An integer error value.
+ *      error: >0 LY_ERR, < 0 -errno
  *	errfmt: An error format string (i.e., printfrr)
  *      ...: args for use by the `errfmt` format string.
  *
