@@ -22,7 +22,7 @@
 #define __log_err(fmt, ...) zlog_err("%s: ERROR: " fmt, __func__, ##__VA_ARGS__)
 
 #define MGMTD_TXN_LOCK(txn)   mgmt_txn_lock(txn, __FILE__, __LINE__)
-#define MGMTD_TXN_UNLOCK(txn, in_hash_free) mgmt_txn_unlock(txn, in_hash_free, __FILE__, __LINE__)
+#define MGMTD_TXN_UNLOCK(txn) mgmt_txn_unlock(txn, __FILE__, __LINE__)
 
 enum mgmt_txn_event {
 	MGMTD_TXN_PROC_SETCFG = 1,
@@ -94,6 +94,7 @@ DECLARE_LIST(mgmt_txn_batches, struct mgmt_txn_be_cfg_batch, list_linkage);
 
 struct mgmt_edit_req {
 	char xpath_created[XPATH_MAXLEN];
+	bool created;
 	bool unlock;
 };
 
@@ -293,7 +294,7 @@ static inline const char *mgmt_txn_commit_phase_str(struct mgmt_txn_ctx *txn)
 }
 
 static void mgmt_txn_lock(struct mgmt_txn_ctx *txn, const char *file, int line);
-static void mgmt_txn_unlock(struct mgmt_txn_ctx **txn, bool in_hash_free, const char *file,
+static void mgmt_txn_unlock(struct mgmt_txn_ctx **txn, const char *file,
 			    int line);
 static int mgmt_txn_send_be_txn_delete(struct mgmt_txn_ctx *txn,
 				       struct mgmt_be_client_adapter *adapter);
@@ -355,7 +356,7 @@ static void mgmt_txn_cfg_batch_free(struct mgmt_txn_be_cfg_batch **batch)
 		}
 	}
 
-	MGMTD_TXN_UNLOCK(&(*batch)->txn, false);
+	MGMTD_TXN_UNLOCK(&(*batch)->txn);
 
 	XFREE(MTYPE_MGMTD_TXN_CFG_BATCH, *batch);
 	*batch = NULL;
@@ -550,7 +551,7 @@ static void mgmt_txn_req_free(struct mgmt_txn_req **txn_req)
 		      (*txn_req)->req_id, mgmt_txn_reqs_count(req_list));
 	}
 
-	MGMTD_TXN_UNLOCK(&(*txn_req)->txn, false);
+	MGMTD_TXN_UNLOCK(&(*txn_req)->txn);
 	XFREE(MTYPE_MGMTD_TXN_REQ, (*txn_req));
 	*txn_req = NULL;
 }
@@ -741,6 +742,8 @@ static int mgmt_txn_send_commit_cfg_reply(struct mgmt_txn_ctx *txn,
 					    txn->commit_cfg_req->req.commit_cfg
 						    .edit->unlock,
 					    true,
+					    txn->commit_cfg_req->req.commit_cfg
+						    .edit->created,
 					    txn->commit_cfg_req->req.commit_cfg
 						    .edit->xpath_created,
 					    success ? 0 : -1,
@@ -1335,7 +1338,8 @@ static int txn_get_tree_data_done(struct mgmt_txn_ctx *txn,
 			  " req_id %" PRIu64 " to requested type %u",
 			  txn->txn_id, req_id, get_tree->result_type);
 
-		(void)mgmt_fe_adapter_txn_error(txn->txn_id, req_id, false, ret,
+		(void)mgmt_fe_adapter_txn_error(txn->txn_id, req_id, false,
+						errno_from_nb_error(ret),
 						"Error converting results of GETTREE");
 	}
 
@@ -1351,7 +1355,7 @@ static int txn_rpc_done(struct mgmt_txn_ctx *txn, struct mgmt_txn_req *txn_req)
 	EVENT_OFF(txn->rpc_timeout);
 
 	if (rpc->errstr)
-		mgmt_fe_adapter_txn_error(txn->txn_id, req_id, false, -1,
+		mgmt_fe_adapter_txn_error(txn->txn_id, req_id, false, -EINVAL,
 					  rpc->errstr);
 	else if (mgmt_fe_adapter_send_rpc_reply(txn->session_id, txn->txn_id,
 						req_id, rpc->result_type,
@@ -1360,7 +1364,8 @@ static int txn_rpc_done(struct mgmt_txn_ctx *txn, struct mgmt_txn_req *txn_req)
 			  " req_id %" PRIu64 " to requested type %u",
 			  txn->txn_id, req_id, rpc->result_type);
 
-		(void)mgmt_fe_adapter_txn_error(txn->txn_id, req_id, false, -1,
+		(void)mgmt_fe_adapter_txn_error(txn->txn_id, req_id, false,
+						-EINVAL,
 						"Error converting results of RPC");
 	}
 
@@ -1830,9 +1835,9 @@ static struct mgmt_txn_ctx *mgmt_txn_create_new(uint64_t session_id,
 	return txn;
 }
 
-static void mgmt_txn_delete(struct mgmt_txn_ctx **txn, bool in_hash_free)
+static void mgmt_txn_delete(struct mgmt_txn_ctx **txn)
 {
-	MGMTD_TXN_UNLOCK(txn, in_hash_free);
+	MGMTD_TXN_UNLOCK(txn);
 }
 
 static unsigned int mgmt_txn_hash_key(const void *data)
@@ -1855,7 +1860,7 @@ static void mgmt_txn_hash_free(void *data)
 {
 	struct mgmt_txn_ctx *txn = data;
 
-	mgmt_txn_delete(&txn, true);
+	mgmt_txn_delete(&txn);
 }
 
 static void mgmt_txn_hash_init(void)
@@ -1905,7 +1910,8 @@ static void mgmt_txn_lock(struct mgmt_txn_ctx *txn, const char *file, int line)
 	      mgmt_txn_type2str(txn->type), txn->txn_id, txn->refcount);
 }
 
-static void mgmt_txn_unlock(struct mgmt_txn_ctx **txn, bool in_hash_free, const char *file, int line)
+static void mgmt_txn_unlock(struct mgmt_txn_ctx **txn, const char *file,
+			    int line)
 {
 	assert(*txn && (*txn)->refcount);
 
@@ -1921,9 +1927,7 @@ static void mgmt_txn_unlock(struct mgmt_txn_ctx **txn, bool in_hash_free, const
 		EVENT_OFF((*txn)->proc_comm_cfg);
 		EVENT_OFF((*txn)->comm_cfg_timeout);
 		EVENT_OFF((*txn)->get_tree_timeout);
-		if (!in_hash_free)
-			hash_release(mgmt_txn_mm->txn_hash, *txn);
-
+		hash_release(mgmt_txn_mm->txn_hash, *txn);
 		mgmt_txns_del(&mgmt_txn_mm->txn_list, *txn);
 
 		__dbg("Deleted %s txn-id: %" PRIu64 " session-id: %" PRIu64,
@@ -1940,7 +1944,7 @@ static void mgmt_txn_cleanup_txn(struct mgmt_txn_ctx **txn)
 {
 	/* TODO: Any other cleanup applicable */
 
-	mgmt_txn_delete(txn, false);
+	mgmt_txn_delete(txn);
 }
 
 static void mgmt_txn_cleanup_all_txns(void)
@@ -2032,7 +2036,7 @@ void mgmt_destroy_txn(uint64_t *txn_id)
 	if (!txn)
 		return;
 
-	mgmt_txn_delete(&txn, false);
+	mgmt_txn_delete(&txn);
 	*txn_id = MGMTD_TXN_ID_NONE;
 }
 
@@ -2565,8 +2569,8 @@ int mgmt_txn_send_edit(uint64_t txn_id, uint64_t req_id,
 	assert(nb_config);
 
 	ret = nb_candidate_edit_tree(nb_config, operation, request_type, xpath,
-				     data, edit->xpath_created, errstr,
-				     sizeof(errstr));
+				     data, &edit->created, edit->xpath_created,
+				     errstr, sizeof(errstr));
 	if (ret)
 		goto reply;
 
@@ -2580,8 +2584,9 @@ int mgmt_txn_send_edit(uint64_t txn_id, uint64_t req_id,
 	}
 reply:
 	mgmt_fe_adapter_send_edit_reply(txn->session_id, txn->txn_id, req_id,
-					unlock, commit, edit->xpath_created,
-					ret ? -1 : 0, errstr);
+					unlock, commit, edit->created,
+					edit->xpath_created,
+					errno_from_nb_error(ret), errstr);
 
 	XFREE(MTYPE_MGMTD_TXN_REQ, edit);
 
@@ -2711,7 +2716,7 @@ int mgmt_txn_notify_error(struct mgmt_be_client_adapter *adapter,
 	case MGMTD_TXN_PROC_GETCFG:
 	case MGMTD_TXN_COMMITCFG_TIMEOUT:
 	default:
-		assert(!"non-native req event in native erorr path");
+		assert(!"non-native req event in native error path");
 		return -1;
 	}
 }
