@@ -1,24 +1,9 @@
+// SPDX-License-Identifier: GPL-2.0-or-later
 /*
  * Route map function of ospfd.
  * Copyright (C) 2000 IP Infusion Inc.
  *
  * Written by Toshiaki Takada.
- *
- * This file is part of GNU Zebra.
- *
- * GNU Zebra is free software; you can redistribute it and/or modify it
- * under the terms of the GNU General Public License as published by the
- * Free Software Foundation; either version 2, or (at your option) any
- * later version.
- *
- * GNU Zebra is distributed in the hope that it will be useful, but
- * WITHOUT ANY WARRANTY; without even the implied warranty of
- * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
- * General Public License for more details.
- *
- * You should have received a copy of the GNU General Public License along
- * with this program; see the file COPYING; if not, write to the Free Software
- * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA
  */
 
 #include <zebra.h>
@@ -33,6 +18,7 @@
 #include "plist.h"
 #include "vrf.h"
 #include "frrstr.h"
+#include "northbound_cli.h"
 
 #include "ospfd/ospfd.h"
 #include "ospfd/ospf_asbr.h"
@@ -40,6 +26,7 @@
 #include "ospfd/ospf_lsa.h"
 #include "ospfd/ospf_route.h"
 #include "ospfd/ospf_zebra.h"
+#include "ospfd/ospf_errors.h"
 
 /* Hook function for updating route_map assignment. */
 static void ospf_route_map_update(const char *name)
@@ -69,11 +56,14 @@ static void ospf_route_map_update(const char *name)
 					/* Keep old route-map. */
 					struct route_map *old = ROUTEMAP(red);
 
-					/* Update route-map. */
 					ROUTEMAP(red) =
 						route_map_lookup_by_name(
 							ROUTEMAP_NAME(red));
 
+					if (!old)
+						route_map_counter_increment(
+							ROUTEMAP(red));
+
 					/* No update for this distribute type.
 					 */
 					if (old == NULL
@@ -88,7 +78,7 @@ static void ospf_route_map_update(const char *name)
 	}
 }
 
-static void ospf_route_map_event(route_map_event_t event, const char *name)
+static void ospf_route_map_event(const char *name)
 {
 	struct ospf *ospf;
 	int type;
@@ -117,29 +107,28 @@ static void ospf_route_map_event(route_map_event_t event, const char *name)
 
 /* `match ip netxthop ' */
 /* Match function return 1 if match is success else return zero. */
-static route_map_result_t route_match_ip_nexthop(void *rule,
-						 const struct prefix *prefix,
-						 route_map_object_t type,
-						 void *object)
+static enum route_map_cmd_result_t
+route_match_ip_nexthop(void *rule, const struct prefix *prefix, void *object)
 {
 	struct access_list *alist;
 	struct external_info *ei = object;
 	struct prefix_ipv4 p;
 
-	if (type == RMAP_OSPF) {
-		p.family = AF_INET;
-		p.prefix = ei->nexthop;
-		p.prefixlen = IPV4_MAX_BITLEN;
+	p.family = AF_INET;
+	p.prefix = ei->nexthop;
+	p.prefixlen = IPV4_MAX_BITLEN;
 
-		alist = access_list_lookup(AFI_IP, (char *)rule);
-		if (alist == NULL)
-			return RMAP_NOMATCH;
-
-		return (access_list_apply(alist, &p) == FILTER_DENY
-				? RMAP_NOMATCH
-				: RMAP_MATCH);
+	alist = access_list_lookup(AFI_IP, (char *)rule);
+	if (alist == NULL) {
+		if (unlikely(CHECK_FLAG(rmap_debug, DEBUG_ROUTEMAP_DETAIL)))
+			zlog_debug(
+				"%s: Access-List Specified: %s does not exist defaulting to NO_MATCH",
+				__func__, (char *)rule);
+		return RMAP_NOMATCH;
 	}
-	return RMAP_NOMATCH;
+
+	return (access_list_apply(alist, &p) == FILTER_DENY ? RMAP_NOMATCH
+							    : RMAP_MATCH);
 }
 
 /* Route map `ip next-hop' match statement. `arg' should be
@@ -156,34 +145,38 @@ static void route_match_ip_nexthop_free(void *rule)
 }
 
 /* Route map commands for metric matching. */
-struct route_map_rule_cmd route_match_ip_nexthop_cmd = {
-	"ip next-hop", route_match_ip_nexthop, route_match_ip_nexthop_compile,
-	route_match_ip_nexthop_free};
+static const struct route_map_rule_cmd route_match_ip_nexthop_cmd = {
+	"ip next-hop",
+	route_match_ip_nexthop,
+	route_match_ip_nexthop_compile,
+	route_match_ip_nexthop_free
+};
 
 /* `match ip next-hop prefix-list PREFIX_LIST' */
 
-static route_map_result_t
+static enum route_map_cmd_result_t
 route_match_ip_next_hop_prefix_list(void *rule, const struct prefix *prefix,
-				    route_map_object_t type, void *object)
+				    void *object)
 {
 	struct prefix_list *plist;
 	struct external_info *ei = object;
 	struct prefix_ipv4 p;
 
-	if (type == RMAP_OSPF) {
-		p.family = AF_INET;
-		p.prefix = ei->nexthop;
-		p.prefixlen = IPV4_MAX_BITLEN;
+	p.family = AF_INET;
+	p.prefix = ei->nexthop;
+	p.prefixlen = IPV4_MAX_BITLEN;
 
-		plist = prefix_list_lookup(AFI_IP, (char *)rule);
-		if (plist == NULL)
-			return RMAP_NOMATCH;
-
-		return (prefix_list_apply(plist, &p) == PREFIX_DENY
-				? RMAP_NOMATCH
-				: RMAP_MATCH);
+	plist = prefix_list_lookup(AFI_IP, (char *)rule);
+	if (plist == NULL) {
+		if (unlikely(CHECK_FLAG(rmap_debug, DEBUG_ROUTEMAP_DETAIL)))
+			zlog_debug(
+				"%s: Prefix List %s specified does not exist defaulting to NO_MATCH",
+				__func__, (char *)rule);
+		return RMAP_NOMATCH;
 	}
-	return RMAP_NOMATCH;
+
+	return (prefix_list_apply(plist, &p) == PREFIX_DENY ? RMAP_NOMATCH
+							    : RMAP_MATCH);
 }
 
 static void *route_match_ip_next_hop_prefix_list_compile(const char *arg)
@@ -196,32 +189,71 @@ static void route_match_ip_next_hop_prefix_list_free(void *rule)
 	XFREE(MTYPE_ROUTE_MAP_COMPILED, rule);
 }
 
-struct route_map_rule_cmd route_match_ip_next_hop_prefix_list_cmd = {
-	"ip next-hop prefix-list", route_match_ip_next_hop_prefix_list,
+static const struct route_map_rule_cmd
+		route_match_ip_next_hop_prefix_list_cmd = {
+	"ip next-hop prefix-list",
+	route_match_ip_next_hop_prefix_list,
 	route_match_ip_next_hop_prefix_list_compile,
-	route_match_ip_next_hop_prefix_list_free};
+	route_match_ip_next_hop_prefix_list_free
+};
+
+/* `match ip next-hop type <blackhole>' */
+
+static enum route_map_cmd_result_t
+route_match_ip_next_hop_type(void *rule, const struct prefix *prefix,
+			     void *object)
+{
+	struct external_info *ei = object;
+
+	if (prefix->family == AF_INET) {
+		ei = (struct external_info *)object;
+		if (!ei)
+			return RMAP_NOMATCH;
+
+		if (ei->nexthop.s_addr == INADDR_ANY && !ei->ifindex)
+			return RMAP_MATCH;
+	}
+	return RMAP_NOMATCH;
+}
+
+static void *route_match_ip_next_hop_type_compile(const char *arg)
+{
+	return XSTRDUP(MTYPE_ROUTE_MAP_COMPILED, arg);
+}
+
+static void route_match_ip_next_hop_type_free(void *rule)
+{
+	XFREE(MTYPE_ROUTE_MAP_COMPILED, rule);
+}
+
+static const struct route_map_rule_cmd
+		route_match_ip_next_hop_type_cmd = {
+	"ip next-hop type",
+	route_match_ip_next_hop_type,
+	route_match_ip_next_hop_type_compile,
+	route_match_ip_next_hop_type_free
+};
 
 /* `match ip address IP_ACCESS_LIST' */
 /* Match function should return 1 if match is success else return
    zero. */
-static route_map_result_t route_match_ip_address(void *rule,
-						 const struct prefix *prefix,
-						 route_map_object_t type,
-						 void *object)
+static enum route_map_cmd_result_t
+route_match_ip_address(void *rule, const struct prefix *prefix, void *object)
 {
 	struct access_list *alist;
 	/* struct prefix_ipv4 match; */
 
-	if (type == RMAP_OSPF) {
-		alist = access_list_lookup(AFI_IP, (char *)rule);
-		if (alist == NULL)
-			return RMAP_NOMATCH;
-
-		return (access_list_apply(alist, prefix) == FILTER_DENY
-				? RMAP_NOMATCH
-				: RMAP_MATCH);
+	alist = access_list_lookup(AFI_IP, (char *)rule);
+	if (alist == NULL) {
+		if (unlikely(CHECK_FLAG(rmap_debug, DEBUG_ROUTEMAP_DETAIL)))
+			zlog_debug(
+				"%s: Access-List Specified: %s does not exist defaulting to NO_MATCH",
+				__func__, (char *)rule);
+		return RMAP_NOMATCH;
 	}
-	return RMAP_NOMATCH;
+
+	return (access_list_apply(alist, prefix) == FILTER_DENY ? RMAP_NOMATCH
+								: RMAP_MATCH);
 }
 
 /* Route map `ip address' match statement.  `arg' should be
@@ -238,27 +270,32 @@ static void route_match_ip_address_free(void *rule)
 }
 
 /* Route map commands for ip address matching. */
-struct route_map_rule_cmd route_match_ip_address_cmd = {
-	"ip address", route_match_ip_address, route_match_ip_address_compile,
-	route_match_ip_address_free};
+static const struct route_map_rule_cmd route_match_ip_address_cmd = {
+	"ip address",
+	route_match_ip_address,
+	route_match_ip_address_compile,
+	route_match_ip_address_free
+};
 
 /* `match ip address prefix-list PREFIX_LIST' */
-static route_map_result_t
+static enum route_map_cmd_result_t
 route_match_ip_address_prefix_list(void *rule, const struct prefix *prefix,
-				   route_map_object_t type, void *object)
+				   void *object)
 {
 	struct prefix_list *plist;
 
-	if (type == RMAP_OSPF) {
-		plist = prefix_list_lookup(AFI_IP, (char *)rule);
-		if (plist == NULL)
-			return RMAP_NOMATCH;
+	plist = prefix_list_lookup(AFI_IP, (char *)rule);
+	if (plist == NULL) {
+		if (unlikely(CHECK_FLAG(rmap_debug, DEBUG_ROUTEMAP_DETAIL)))
+			zlog_debug(
+				"%s: Prefix List %s specified does not exist defaulting to NO_MATCH",
+				__func__, (char *)rule);
 
-		return (prefix_list_apply(plist, prefix) == PREFIX_DENY
-				? RMAP_NOMATCH
-				: RMAP_MATCH);
+		return RMAP_NOMATCH;
 	}
-	return RMAP_NOMATCH;
+
+	return (prefix_list_apply(plist, prefix) == PREFIX_DENY ? RMAP_NOMATCH
+								: RMAP_MATCH);
 }
 
 static void *route_match_ip_address_prefix_list_compile(const char *arg)
@@ -271,32 +308,30 @@ static void route_match_ip_address_prefix_list_free(void *rule)
 	XFREE(MTYPE_ROUTE_MAP_COMPILED, rule);
 }
 
-struct route_map_rule_cmd route_match_ip_address_prefix_list_cmd = {
-	"ip address prefix-list", route_match_ip_address_prefix_list,
+static const struct route_map_rule_cmd
+		route_match_ip_address_prefix_list_cmd = {
+	"ip address prefix-list",
+	route_match_ip_address_prefix_list,
 	route_match_ip_address_prefix_list_compile,
-	route_match_ip_address_prefix_list_free};
+	route_match_ip_address_prefix_list_free
+};
 
 /* `match interface IFNAME' */
 /* Match function should return 1 if match is success else return
    zero. */
-static route_map_result_t route_match_interface(void *rule,
-						const struct prefix *prefix,
-						route_map_object_t type,
-						void *object)
+static enum route_map_cmd_result_t
+route_match_interface(void *rule, const struct prefix *prefix, void *object)
 {
 	struct interface *ifp;
 	struct external_info *ei;
 
-	if (type == RMAP_OSPF) {
-		ei = object;
-		ifp = if_lookup_by_name_all_vrf((char *)rule);
+	ei = object;
+	ifp = if_lookup_by_name((char *)rule, ei->ospf->vrf_id);
 
-		if (ifp == NULL || ifp->ifindex != ei->ifindex)
-			return RMAP_NOMATCH;
+	if (ifp == NULL || ifp->ifindex != ei->ifindex)
+		return RMAP_NOMATCH;
 
-		return RMAP_MATCH;
-	}
-	return RMAP_NOMATCH;
+	return RMAP_MATCH;
 }
 
 /* Route map `interface' match statement.  `arg' should be
@@ -313,31 +348,31 @@ static void route_match_interface_free(void *rule)
 }
 
 /* Route map commands for ip address matching. */
-struct route_map_rule_cmd route_match_interface_cmd = {
-	"interface", route_match_interface, route_match_interface_compile,
-	route_match_interface_free};
+static const struct route_map_rule_cmd route_match_interface_cmd = {
+	"interface",
+	route_match_interface,
+	route_match_interface_compile,
+	route_match_interface_free
+};
 
 /* Match function return 1 if match is success else return zero. */
-static route_map_result_t route_match_tag(void *rule,
-					  const struct prefix *prefix,
-					  route_map_object_t type, void *object)
+static enum route_map_cmd_result_t
+route_match_tag(void *rule, const struct prefix *prefix, void *object)
 {
 	route_tag_t *tag;
 	struct external_info *ei;
 
-	if (type == RMAP_OSPF) {
-		tag = rule;
-		ei = object;
+	tag = rule;
+	ei = object;
 
-		return ((ei->tag == *tag) ? RMAP_MATCH : RMAP_NOMATCH);
-	}
-
-	return RMAP_NOMATCH;
+	return ((ei->tag == *tag) ? RMAP_MATCH : RMAP_NOMATCH);
 }
 
 /* Route map commands for tag matching. */
-static struct route_map_rule_cmd route_match_tag_cmd = {
-	"tag", route_match_tag, route_map_rule_tag_compile,
+static const struct route_map_rule_cmd route_match_tag_cmd = {
+	"tag",
+	route_match_tag,
+	route_map_rule_tag_compile,
 	route_map_rule_tag_free,
 };
 
@@ -349,34 +384,34 @@ struct ospf_metric {
 
 /* `set metric METRIC' */
 /* Set metric to attribute. */
-static route_map_result_t route_set_metric(void *rule,
-					   const struct prefix *prefix,
-					   route_map_object_t type,
-					   void *object)
+static enum route_map_cmd_result_t
+route_set_metric(void *rule, const struct prefix *prefix, void *object)
 {
 	struct ospf_metric *metric;
 	struct external_info *ei;
 
-	if (type == RMAP_OSPF) {
-		/* Fetch routemap's rule information. */
-		metric = rule;
-		ei = object;
+	/* Fetch routemap's rule information. */
+	metric = rule;
+	ei = object;
 
-		/* Set metric out value. */
-		if (!metric->used)
-			return RMAP_OKAY;
-		if (metric->type == metric_increment)
-			ei->route_map_set.metric += metric->metric;
-		if (metric->type == metric_decrement)
-			ei->route_map_set.metric -= metric->metric;
-		if (metric->type == metric_absolute)
-			ei->route_map_set.metric = metric->metric;
+	/* Set metric out value. */
+	if (!metric->used)
+		return RMAP_OKAY;
+
+	ROUTEMAP_METRIC(ei) = ei->metric;
+
+	if (metric->type == metric_increment)
+		ROUTEMAP_METRIC(ei) += metric->metric;
+	else if (metric->type == metric_decrement)
+		ROUTEMAP_METRIC(ei) -= metric->metric;
+	else if (metric->type == metric_absolute)
+		ROUTEMAP_METRIC(ei) = metric->metric;
+
+	if ((uint32_t)ROUTEMAP_METRIC(ei) < ei->min_metric)
+		ROUTEMAP_METRIC(ei) = ei->min_metric;
+	if ((uint32_t)ROUTEMAP_METRIC(ei) > ei->max_metric)
+		ROUTEMAP_METRIC(ei) = ei->max_metric;
 
-		if ((signed int)ei->route_map_set.metric < 1)
-			ei->route_map_set.metric = -1;
-		if (ei->route_map_set.metric > OSPF_LS_INFINITY)
-			ei->route_map_set.metric = OSPF_LS_INFINITY;
-	}
 	return RMAP_OKAY;
 }
 
@@ -385,14 +420,15 @@ static void *route_set_metric_compile(const char *arg)
 {
 	struct ospf_metric *metric;
 
-	metric = XCALLOC(MTYPE_ROUTE_MAP_COMPILED, sizeof(uint32_t));
+	metric = XCALLOC(MTYPE_ROUTE_MAP_COMPILED, sizeof(*metric));
 	metric->used = false;
 
 	if (all_digit(arg))
 		metric->type = metric_absolute;
 
 	if (strmatch(arg, "+rtt") || strmatch(arg, "-rtt")) {
-		zlog_warn("OSPF does not support 'set metric +rtt / -rtt'");
+		flog_warn(EC_OSPF_SET_METRIC_PLUS,
+			  "OSPF does not support 'set metric +rtt / -rtt'");
 		return metric;
 	}
 
@@ -421,29 +457,137 @@ static void route_set_metric_free(void *rule)
 }
 
 /* Set metric rule structure. */
-struct route_map_rule_cmd route_set_metric_cmd = {
-	"metric", route_set_metric, route_set_metric_compile,
+static const struct route_map_rule_cmd route_set_metric_cmd = {
+	"metric",
+	route_set_metric,
+	route_set_metric_compile,
 	route_set_metric_free,
 };
 
+/* `set min-metric METRIC' */
+/* Set min-metric to attribute. */
+static enum route_map_cmd_result_t
+route_set_min_metric(void *rule, const struct prefix *prefix, void *object)
+{
+	uint32_t *min_metric;
+	struct external_info *ei;
+
+	/* Fetch routemap's rule information. */
+	min_metric = rule;
+	ei = object;
+
+	ei->min_metric = *min_metric;
+
+	if (ei->min_metric > OSPF_LS_INFINITY)
+		ei->min_metric = OSPF_LS_INFINITY;
+
+	if ((uint32_t)ROUTEMAP_METRIC(ei) < ei->min_metric)
+		ROUTEMAP_METRIC(ei) = ei->min_metric;
+
+	return RMAP_OKAY;
+}
+
+/* set min-metric compilation. */
+static void *route_set_min_metric_compile(const char *arg)
+{
+
+	uint32_t *min_metric;
+
+	min_metric = XCALLOC(MTYPE_ROUTE_MAP_COMPILED, sizeof(uint32_t));
+
+	*min_metric = strtoul(arg, NULL, 10);
+
+	if (*min_metric)
+		return min_metric;
+
+	XFREE(MTYPE_ROUTE_MAP_COMPILED, min_metric);
+	return NULL;
+}
+
+/* Free route map's compiled `set min-metric' value. */
+static void route_set_min_metric_free(void *rule)
+{
+	XFREE(MTYPE_ROUTE_MAP_COMPILED, rule);
+}
+
+/* Set metric rule structure. */
+static const struct route_map_rule_cmd route_set_min_metric_cmd = {
+	"min-metric",
+	route_set_min_metric,
+	route_set_min_metric_compile,
+	route_set_min_metric_free,
+};
+
+
+/* `set max-metric METRIC' */
+/* Set max-metric to attribute. */
+static enum route_map_cmd_result_t
+route_set_max_metric(void *rule, const struct prefix *prefix, void *object)
+{
+	uint32_t *max_metric;
+	struct external_info *ei;
+
+	/* Fetch routemap's rule information. */
+	max_metric = rule;
+	ei = object;
+
+	ei->max_metric = *max_metric;
+
+	if (ei->max_metric > OSPF_LS_INFINITY)
+		ei->max_metric = OSPF_LS_INFINITY;
+
+	if ((uint32_t)ROUTEMAP_METRIC(ei) > ei->max_metric)
+		ROUTEMAP_METRIC(ei) = ei->max_metric;
+
+	return RMAP_OKAY;
+}
+
+/* set max-metric compilation. */
+static void *route_set_max_metric_compile(const char *arg)
+{
+
+	uint32_t *max_metric;
+
+	max_metric = XCALLOC(MTYPE_ROUTE_MAP_COMPILED, sizeof(uint32_t));
+
+	*max_metric = strtoul(arg, NULL, 10);
+
+	if (*max_metric)
+		return max_metric;
+
+	XFREE(MTYPE_ROUTE_MAP_COMPILED, max_metric);
+	return NULL;
+}
+
+/* Free route map's compiled `set max-metric' value. */
+static void route_set_max_metric_free(void *rule)
+{
+	XFREE(MTYPE_ROUTE_MAP_COMPILED, rule);
+}
+
+/* Set metric rule structure. */
+static const struct route_map_rule_cmd route_set_max_metric_cmd = {
+	"max-metric",
+	route_set_max_metric,
+	route_set_max_metric_compile,
+	route_set_max_metric_free,
+};
+
 /* `set metric-type TYPE' */
 /* Set metric-type to attribute. */
-static route_map_result_t route_set_metric_type(void *rule,
-						const struct prefix *prefix,
-						route_map_object_t type,
-						void *object)
+static enum route_map_cmd_result_t
+route_set_metric_type(void *rule, const struct prefix *prefix, void *object)
 {
 	uint32_t *metric_type;
 	struct external_info *ei;
 
-	if (type == RMAP_OSPF) {
-		/* Fetch routemap's rule information. */
-		metric_type = rule;
-		ei = object;
+	/* Fetch routemap's rule information. */
+	metric_type = rule;
+	ei = object;
+
+	/* Set metric out value. */
+	ROUTEMAP_METRIC_TYPE(ei) = *metric_type;
 
-		/* Set metric out value. */
-		ei->route_map_set.metric_type = *metric_type;
-	}
 	return RMAP_OKAY;
 }
 
@@ -473,35 +617,37 @@ static void route_set_metric_type_free(void *rule)
 }
 
 /* Set metric rule structure. */
-struct route_map_rule_cmd route_set_metric_type_cmd = {
-	"metric-type", route_set_metric_type, route_set_metric_type_compile,
+static const struct route_map_rule_cmd route_set_metric_type_cmd = {
+	"metric-type",
+	route_set_metric_type,
+	route_set_metric_type_compile,
 	route_set_metric_type_free,
 };
 
-static route_map_result_t route_set_tag(void *rule, const struct prefix *prefix,
-					route_map_object_t type, void *object)
+static enum route_map_cmd_result_t
+route_set_tag(void *rule, const struct prefix *prefix, void *object)
 {
 	route_tag_t *tag;
 	struct external_info *ei;
 
-	if (type == RMAP_OSPF) {
-		tag = rule;
-		ei = object;
+	tag = rule;
+	ei = object;
 
-		/* Set tag value */
-		ei->tag = *tag;
-	}
+	/* Set tag value */
+	ei->tag = *tag;
 
 	return RMAP_OKAY;
 }
 
 /* Route map commands for tag set. */
-static struct route_map_rule_cmd route_set_tag_cmd = {
-	"tag", route_set_tag, route_map_rule_tag_compile,
+static const struct route_map_rule_cmd route_set_tag_cmd = {
+	"tag",
+	route_set_tag,
+	route_map_rule_tag_compile,
 	route_map_rule_tag_free,
 };
 
-DEFUN (set_metric_type,
+DEFUN_YANG (set_metric_type,
        set_metric_type_cmd,
        "set metric-type <type-1|type-2>",
        SET_STR
@@ -510,11 +656,19 @@ DEFUN (set_metric_type,
        "OSPF[6] external type 2 metric\n")
 {
 	char *ext = argv[2]->text;
-	return generic_set_add(vty, VTY_GET_CONTEXT(route_map_index),
-			       "metric-type", ext);
+
+	const char *xpath =
+		"./set-action[action='frr-ospf-route-map:metric-type']";
+	char xpath_value[XPATH_MAXLEN];
+
+	nb_cli_enqueue_change(vty, xpath, NB_OP_CREATE, NULL);
+	snprintf(xpath_value, sizeof(xpath_value),
+		 "%s/rmap-set-action/frr-ospf-route-map:metric-type", xpath);
+	nb_cli_enqueue_change(vty, xpath_value, NB_OP_MODIFY, ext);
+	return nb_cli_apply_changes(vty, NULL);
 }
 
-DEFUN (no_set_metric_type,
+DEFUN_YANG (no_set_metric_type,
        no_set_metric_type_cmd,
        "no set metric-type [<type-1|type-2>]",
        NO_STR
@@ -523,9 +677,11 @@ DEFUN (no_set_metric_type,
        "OSPF[6] external type 1 metric\n"
        "OSPF[6] external type 2 metric\n")
 {
-	char *ext = (argc == 4) ? argv[3]->text : NULL;
-	return generic_set_delete(vty, VTY_GET_CONTEXT(route_map_index),
-				  "metric-type", ext);
+	const char *xpath =
+		"./set-action[action='frr-ospf-route-map:metric-type']";
+
+	nb_cli_enqueue_change(vty, xpath, NB_OP_DESTROY, NULL);
+	return nb_cli_apply_changes(vty, NULL);
 }
 
 /* Route-map init */
@@ -537,9 +693,6 @@ void ospf_route_map_init(void)
 	route_map_delete_hook(ospf_route_map_update);
 	route_map_event_hook(ospf_route_map_event);
 
-	route_map_set_metric_hook(generic_set_add);
-	route_map_no_set_metric_hook(generic_set_delete);
-
 	route_map_match_ip_next_hop_hook(generic_match_add);
 	route_map_no_match_ip_next_hop_hook(generic_match_delete);
 
@@ -555,12 +708,21 @@ void ospf_route_map_init(void)
 	route_map_match_ip_next_hop_prefix_list_hook(generic_match_add);
 	route_map_no_match_ip_next_hop_prefix_list_hook(generic_match_delete);
 
+	route_map_match_ip_next_hop_type_hook(generic_match_add);
+	route_map_no_match_ip_next_hop_type_hook(generic_match_delete);
+
 	route_map_match_tag_hook(generic_match_add);
 	route_map_no_match_tag_hook(generic_match_delete);
 
 	route_map_set_metric_hook(generic_set_add);
 	route_map_no_set_metric_hook(generic_set_delete);
 
+	route_map_set_min_metric_hook(generic_set_add);
+	route_map_no_set_min_metric_hook(generic_set_delete);
+
+	route_map_set_max_metric_hook(generic_set_add);
+	route_map_no_set_max_metric_hook(generic_set_delete);
+
 	route_map_set_tag_hook(generic_set_add);
 	route_map_no_set_tag_hook(generic_set_delete);
 
@@ -568,10 +730,13 @@ void ospf_route_map_init(void)
 	route_map_install_match(&route_match_ip_next_hop_prefix_list_cmd);
 	route_map_install_match(&route_match_ip_address_cmd);
 	route_map_install_match(&route_match_ip_address_prefix_list_cmd);
+	route_map_install_match(&route_match_ip_next_hop_type_cmd);
 	route_map_install_match(&route_match_interface_cmd);
 	route_map_install_match(&route_match_tag_cmd);
 
 	route_map_install_set(&route_set_metric_cmd);
+	route_map_install_set(&route_set_min_metric_cmd);
+	route_map_install_set(&route_set_max_metric_cmd);
 	route_map_install_set(&route_set_metric_type_cmd);
 	route_map_install_set(&route_set_tag_cmd);
 
