@@ -62,7 +62,6 @@ static void gm_sg_timer_start(struct gm_if *gm_ifp, struct gm_sg *sg,
 		sg->iface->ifp->name, &sg->sgaddr
 
 /* clang-format off */
-#if PIM_IPV == 6
 static const pim_addr gm_all_hosts = {
 	.s6_addr = {
 		0xff, 0x02, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
@@ -84,13 +83,6 @@ static const pim_addr gm_dummy_untracked = {
 		0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff,
 	},
 };
-#else
-/* 224.0.0.1 */
-static const pim_addr gm_all_hosts = { .s_addr = htonl(0xe0000001), };
-/* 224.0.0.22 */
-static const pim_addr gm_all_routers = { .s_addr = htonl(0xe0000016), };
-static const pim_addr gm_dummy_untracked = { .s_addr = 0xffffffff, };
-#endif
 /* clang-format on */
 
 #define IPV6_MULTICAST_SCOPE_LINK 2
@@ -1665,6 +1657,18 @@ static void gm_t_recv(struct event *t)
 		goto out_free;
 	}
 
+	struct interface *ifp;
+
+	ifp = if_lookup_by_index(pkt_src->sin6_scope_id, pim->vrf->vrf_id);
+	if (!ifp || !ifp->info)
+		goto out_free;
+
+	struct pim_interface *pim_ifp = ifp->info;
+	struct gm_if *gm_ifp = pim_ifp->mld;
+
+	if (!gm_ifp)
+		goto out_free;
+
 	for (cmsg = CMSG_FIRSTHDR(mh); cmsg; cmsg = CMSG_NXTHDR(mh, cmsg)) {
 		if (cmsg->cmsg_level != SOL_IPV6)
 			continue;
@@ -1683,21 +1687,6 @@ static void gm_t_recv(struct event *t)
 		}
 	}
 
-	/* Prefer pktinfo as that also works in case of VRF */
-	ifindex_t ifindex = pktinfo ? pktinfo->ipi6_ifindex
-	                            : pkt_src->sin6_scope_id;
-	struct interface *ifp;
-
-	ifp = if_lookup_by_index(ifindex, pim->vrf->vrf_id);
-	if (!ifp || !ifp->info)
-		goto out_free;
-
-	struct pim_interface *pim_ifp = ifp->info;
-	struct gm_if *gm_ifp = pim_ifp->mld;
-
-	if (!gm_ifp)
-		goto out_free;
-
 	if (!pktinfo || !hoplimit) {
 		zlog_err(log_ifp(
 			"BUG: packet without IPV6_PKTINFO or IPV6_HOPLIMIT"));
@@ -2540,7 +2529,7 @@ static void gm_show_if_vrf(struct vty *vty, struct vrf *vrf, const char *ifname,
 	if (!js && !detail) {
 		table = ttable_dump(tt, "\n");
 		vty_out(vty, "%s\n", table);
-		XFREE(MTYPE_TMP, table);
+		XFREE(MTYPE_TMP_TTABLE, table);
 		ttable_del(tt);
 	}
 }
@@ -3024,7 +3013,7 @@ static void gm_show_groups(struct vty *vty, struct vrf *vrf, bool uj)
 		/* Dump the generated table. */
 		table = ttable_dump(tt, "\n");
 		vty_out(vty, "%s\n", table);
-		XFREE(MTYPE_TMP, table);
+		XFREE(MTYPE_TMP_TTABLE, table);
 		ttable_del(tt);
 	}
 }
